package org.deephacks.confit.admin.query;

import org.deephacks.confit.model.Bean;

import java.util.List;

/**
 * A collection of beans representing a result set, which is generated by executing a BeanQuery.
 * The result set maintains a cursor pointing to the next result.
 *
 * Queries only move forward, never backwards.
 */
public interface BeanQueryResult {
    /**
     * @return beans in this result set.
     */
    public List<Bean> get();

    /**
     * Get the position of the first bean in the next result set.
     *
     * Use this value on {@link BeanQuery#setFirstResult(String)} using
     * same query to fetch the next result set.
     *
     * @return position of the next result set.
     */
    public String nextFirstResult();

}
